
/*MA****************************************************************/
/*                                                                 */
/*     File: lexscore.C                                            */
/*   Author: Helmut Schmid                                         */
/*  Purpose:                                                       */
/*  Created: Mon Jan 27 10:26:12 2003                              */
/* Modified: Thu Jan 22 14:18:06 2009 (schmid)                     */
/*                                                                 */
/*ME****************************************************************/


#include "tree.h"
#include <ext/hash_set>
using __gnu_cxx::hash_set;
using __gnu_cxx::hash;

bool LexScore=false;


/****** ScoreTable *************************************************/

class ScoreTable {

private:
  
  struct eqstr
  {
    bool operator()(const char* s1, const char* s2) const
    {
      return strcmp(s1, s2) == 0;
    }
  };

  typedef hash_map<const char*, double, hash<const char*>, eqstr> Table;
  Table table;

public:
  void add( const char *tuple, double s ) {
    Table::iterator it=table.find( tuple );
    if (it != table.end())
      it->second = s;
    else
      table[strdup(tuple)] = s;
  }

  double score(const char *h, const char *lhs, const char *H, const char *rhs) {
    char buffer[1000];
    // regular lookup
    sprintf(buffer, "%s\t%s %s\t%s\n", h, lhs, H, rhs);
    Table::iterator it = table.find( buffer );
    if (it != table.end())
      return it->second;

    // default score
    sprintf(buffer,"%s\t%s\t<DEFAULT>\n",h,lhs);
    it = table.find( buffer );
    if (it != table.end())
      return it->second;

    // unknown word
    return 1.0;
  }
};

ScoreTable STab;


/*FA****************************************************************/
/*                                                                 */
/*  read_lexscores                                                 */
/*                                                                 */
/*FE****************************************************************/

void read_lexscores( char *filename )

{
  FILE *file=fopen(filename,"rt");
  if (file == NULL) {
    fprintf(stderr,"Error: unable to open file \"%s\"!\n", filename);
    exit(1);
  }
  char buffer[1001];
  double score;
  while (fscanf(file,"%lf ", &score) == 1 && fgets(buffer,1000,file))
    STab.add(buffer, score);
  fclose(file);
}


/*******************************************************************/
/*                                                                 */
/*  append                                                         */
/*                                                                 */
/*******************************************************************/

static char *append( char *p, const char *s )

{
  while (*s)
    *(p++) = *(s++);
  return p;
}


/*FA****************************************************************/
/*                                                                 */
/*  Tree::lexscore                                                 */
/*                                                                 */
/*FE****************************************************************/

double Tree::lexscore()

{
  double score=1.0;

  if (daughters.size() > 0 && 
      (daughters.size() > 1 || daughters[0]->daughters.size() > 0))
    {
      char buffer[1000];
      char *p=buffer;
      
      // rhs of the rule
      for( size_t i=0; i<daughters.size(); i++ ) {
	score *= daughters[i]->lexscore();
	p = append( p, daughters[i]->label);
	*(p++) = ' ';
      }
      *(--p) = 0; // end of string
      // lookup in the score table
      double s = STab.score( head->daughters[0]->label, label, 
			     head_daughter->label, buffer);
      score *= s;

#if 0
      if (s != 1.0)
	fprintf(stdout,"%s\t%s %s\t%s\t%lf\n", head->daughters[0]->label, 
		label, head_daughter->label, buffer, s);
#endif
    }
  
  return score;
}
