
/*MA****************************************************************/
/*                                                                 */
/*     File: rules.C                                               */
/*   Author: Helmut Schmid                                         */
/*  Purpose:                                                       */
/*  Created: Mon Jan 27 10:26:12 2003                              */
/* Modified: Fri Mar  6 11:05:29 2009 (schmid)                     */
/*                                                                 */
/*ME****************************************************************/

#include "tree.h"


/*****************  class FrequencyTable  **************************/

class FrequencyTable {

private:

  struct eqstr {
    bool operator()(const char* s1, const char* s2) const {
      return strcmp(s1, s2) == 0;
    }
  };

  typedef hash_map<char*, long, hash<const char*>, eqstr> SymbolMap;

  SymbolMap ST;

public:
  typedef SymbolMap::iterator iterator;

  ~FrequencyTable() {
    vector<char*> s;
    for( iterator it=ST.begin(); it!=ST.end(); it++ )
      s.push_back(it->first);
    ST.clear();
    for( size_t i=0; i<s.size(); i++ )
      free(s[i]);
  }

  long &operator[]( char *s ) {
    SymbolMap::iterator it=ST.find(s);
    if (it != ST.end())
      return it->second;

    // insert the new symbol into the tables
    char *r = strdup(s);
    return ST.insert(SymbolMap::value_type(r, 0)).first->second;
  }

  iterator find( char *s ) { return ST.find(s); }
  iterator begin() { return ST.begin(); }
  iterator end()   { return ST.end(); }
  size_t   size()  { return ST.size(); }
};


FrequencyTable LF, GF;


/*FA****************************************************************/
/*                                                                 */
/*  add_string                                                     */
/*                                                                 */
/*FE****************************************************************/

static char *add_string( char *p, char *s )

{
  while ((*(p++) = *(s++))) ;
  return p-1;
}


/*FA****************************************************************/
/*                                                                 */
/*  Tree::add_rules                                                */
/*                                                                 */
/*FE****************************************************************/

void Tree::add_rules()

{
  if (daughters.size() > 0) {
    static char buffer[10000];
    char *p = buffer;

    if (daughters.size() == 1 && daughters[0]->daughters.size() == 0) {
      // lexical entry
      sprintf(buffer,"%s %s", label, daughters[0]->label);
      LF[buffer]++;
    }

    else {
      // grammar rule
      if (WithHeads) {
	p = add_string(p, head->daughters[0]->label);
	*(p++) = ' ';
      }
      p = add_string(p, label);
      for( size_t i=0; i<daughters.size(); i++ ) {
	*(p++) = ' ';
	if (WithHeads && daughters[i]->head == head)
	  *(p++) = '^';
	p = add_string(p, daughters[i]->label);
      }
      GF[buffer]++;
    }

    for( size_t i=0; i<daughters.size(); i++ )
      daughters[i]->add_rules();
  }
}


/*FA****************************************************************/
/*                                                                 */
/*  print_grammar                                                  */
/*                                                                 */
/*FE****************************************************************/

void print_grammar()

{
  for( FrequencyTable::iterator it=GF.begin(); it!=GF.end(); it++ )
    fprintf(outfile,"%ld %s\n", it->second, it->first);

  if (lexfile == outfile)
    fputc('\n', outfile);

  for( FrequencyTable::iterator it=LF.begin(); it!=LF.end(); it++ )
    fprintf(lexfile,"%ld %s\n", it->second, it->first);
}
